@@ -8,10 +8,13 @@ from py7za import Py7za, AsyncIOPool, available_cpu_count, nice_size
 # only used for status
 from zipfile import ZipFile
 
+ARCHIVE_SUFFIXES = ['.zip', '.zipx', '.gz', '.7z', '.s7z', '.lzma', '.lz', '.cab', '.jar', '.war', '.rar']
+
 
 async def box(cfg):
     total = 0
     done = False
+    skipped = 0
     total_content_size = 0
     total_zip_size = 0
     current = 0
@@ -23,23 +26,31 @@ async def box(cfg):
     cli_options += f'-ao{cfg.overwrite} ' if cfg['unbox'] else ''
     cli_options += cfg['7za'] if '7za' in cfg else ''
 
-    print_result = cfg.output in 'vd'
+    print_result = cfg.output in 'dlv'
     info_command = cfg.output == 'v'
     if info_command:
         basicConfig(level=INFO, format='%(asctime)s %(levelname)s %(message)s')
     update_status = cfg.output == 's'
+    list_status = cfg.output == 'd'
 
     unbox = cfg['unbox']
+    unbox_multi = cfg['unbox_multi']
     delete = cfg['delete']
     create_folders = cfg['create_folders']
     zip_structure = cfg['zip_structure']
+    zip_archives = cfg['zip_archives']
     si = cfg['si']
 
     def globber(root, glob_expr):
+        nonlocal skipped
         if not isinstance(glob_expr, list):
             glob_expr = [glob_expr]
         for ge in glob_expr:
             for fn in Path(root).glob(ge):
+                if not unbox and not zip_archives and fn.suffix in ARCHIVE_SUFFIXES:
+                    info(f'Skipping {fn} as it is an archive and --zip_archives was not specified.')
+                    skipped += 1
+                    continue
                 if (fn.is_dir() and cfg['match_dir']) or (fn.is_file() and cfg['match_file']):
                     yield fn.relative_to(root).parent, fn.name
 
@@ -54,16 +65,18 @@ async def box(cfg):
         nonlocal done, running, current, total
         while True:
             if total == 0:
-                stdout.write(f'\rStarting ... ')
+                stdout.write(f'\x1b[2K\rStarting ... ')
             else:
-                stdout.write(f'\rProcessing: {current}/{total} ... '
+                stdout.write(f'\x1b[2K\rProcessing: {current} [{nice_size(total_content_size, si)}] '
+                             f'/ {total} [{nice_size(total_zip_size, si)}] ... '
                              f'{" ".join([f"{str(py7za.progress)}/100%" for py7za in running if py7za.progress])}')
+            stdout.flush()
             if done:
                 return
             await sleep(0.5)
 
     async def run_all():
-        nonlocal aiop, total, done, total_content_size, total_zip_size
+        nonlocal aiop, total, done, total_content_size, total_zip_size, skipped
         zippers = []
 
         root = Path(cfg.root).absolute()
@@ -82,8 +95,13 @@ async def box(cfg):
                 zippers.append(
                     Py7za(f'a "{target_path}.zip" "{content}" {cli_options}', on_start=start, working_dir=wd))
             else:
+                archive = root / sub_path / fn
+                if not unbox_multi and len(ZipFile(archive).filelist) > 1:
+                    info(f'Skipping {archive} as it has multiple files and --unbox_multi was not specified.')
+                    skipped += 1
+                    continue
                 target_path = target / sub_path if create_folders else target
-                zippers.append(Py7za(f'x "{root / sub_path / fn}" -o"{target_path}" {cli_options}', start))
+                zippers.append(Py7za(f'x "{archive}" "-o{target_path}" {cli_options}', start))
         total = len(zippers)
         async for py7za in aiop.arun_many(zippers):
             if print_result or update_status:
@@ -91,7 +109,14 @@ async def box(cfg):
                 total_content_size += (cs := sum([zip_info.file_size for zip_info in ZipFile(fn).filelist]))
                 total_zip_size += (zs := stat(fn).st_size)
                 if print_result:
-                    print(f'{nice_size(cs, si)} {"from" if unbox else "into"} {nice_size(zs, si)} {fn}')
+                    if list_status:
+                        stdout.write(f'\x1b[2K\r'
+                                     f'{nice_size(cs, si)} {"from" if unbox else "into"} {nice_size(zs, si)} {fn}\n')
+                        stdout.write(f'Total: {current} [{nice_size(total_content_size, si)}] '
+                                     f'/ {total} [{nice_size(total_zip_size, si)}]')
+                    else:
+                        stdout.write(f'{nice_size(cs, si)} {"from" if unbox else "into"} {nice_size(zs, si)} {fn}\n')
+                    stdout.flush()
             if unbox and delete and (py7za.return_code == 0):
                 os_remove(py7za.arguments[1])
             running.remove(py7za)
@@ -103,10 +128,11 @@ async def box(cfg):
         await run_all()
 
     if cfg.output != 'q':
-        if update_status:
-            stdout.write('\r')
+        if update_status or print_result:
+            stdout.write('\x1b[2K\r')
         print(f'Completed processing {nice_size(total_content_size)} '
-              f'of files {"from" if unbox else "into"} {total} archives, totaling {nice_size(total_zip_size)}.')
+              f'of files {"from" if unbox else "into"} {total} archives, totaling {nice_size(total_zip_size)}.'
+              f'\nSkipped {skipped} matched files.')
 
 
 def print_help():
@@ -115,9 +141,9 @@ def print_help():
         '\npy7za-box '+__version__+', command line utility\n'
         '\nPy7za-box ("pizza box") replaces a set of files with individual .zip files\n'
         'containing the originals, or does the reverse by "unboxing" the archives.\n'
-        'Py7za uses 7za.exe, more information on the project page.'
+        'Py7za uses 7za.exe, more information on the project page.\n'
         '\n'
-        'Use: `py7za-box <glob expression(s)> [options] [7za options]\n'
+        'Usage: `py7za-box <glob expression(s)> [options]`\n'
         '\n'
         '<glob expression(s)>      : Glob expression(s) like "**/*.csv". (required)\n'
         '                            Add quotes if your expression contains spaces.\n'
@@ -131,12 +157,15 @@ def print_help():
         '-r/--root <path>          : Path glob expression(s) are relative to. ["."]\n'
         '-t/--target <path>        : Root path for output. ["" / in-place]\n'
         '-u/--unbox/--unzip        : Unzip instead of zip (glob to match archives).\n'
-        '-o/--output [d/q/s/v]     : Default (a line per archive), quiet, status, or\n'
-        '                            verbose output. Verbose prints each 7za command.\n'
-        '                            Note: d/v may malfunction on some archive types.\n'
+        '-um/--unbox_multi         : Whether to unzip multi-file archives. [False]\n'
+        '-o/--output [d/l/q/s/v]   : Default (a line per archive with status), list,\n'
+        '                            quiet, status, or verbose output. Verbose prints\n'
+        '                            each full 7za command.\n'
+        '                            Note: d/l/v does not work for all archive types.\n'
         '-si                       : Whether to use SI units for file sizes. [False]\n'
         '-w/overwrite [a/s/u/t]    : Used overwrite mode when unboxing. [s]\n'
         '                            a:all, s:skip, u:rename new, t:rename existing.\n'
+        '-za/--zip_archives [bool] : Whether to zip matched archives (again). [False]\n'
         '-zs/--zip_structure [bool]: Root sub-folder structure is archived. [False]\n'
         '-7/--7za                  : CLI arguments passed to 7za after scripted ones.\n'
         '                            Add quotes if passing more than one argument.\n'
@@ -163,8 +192,10 @@ CLI_DEFAULTS = {
     'si': False,
     'root': '.',
     'unbox': False,
+    'unbox_multi': False,
     'verbose': False,
     'zip_structure': False,
+    'zip_archives': False,
     '7za': ''
 }
 
@@ -173,7 +204,7 @@ def cli_entry_point():
     cfg = Config.startup(defaults=CLI_DEFAULTS, aliases={
         'h': 'help', 'c': 'cores', 'cf': 'create_folders', 'md': 'match_dir', 'mf': 'match_file', 'u': 'unbox',
         'unzip': 'unbox', 'r': 'root', 'zs': 'zip_structure', 't': 'target', 'v': 'verbose', '7': '7za', 'g': 'glob',
-        'o': 'output', 'w': 'overwrite'
+        'o': 'output', 'w': 'overwrite', 'za': 'zip_archives', 'um': 'unbox_multi'
     })
 
     if cfg.get_as_type('help', bool, False):
@@ -201,7 +232,13 @@ def cli_entry_point():
                 f'unless sub-folder from root were included when the archives were created.')
 
     if cfg.zip_structure and cfg.unbox:
-        warning(f'The --zip_structure option was specified, but does not do anything when unboxing and will ignored.')
+        warning(f'The --zip_structure option was specified, but does nothing when unboxing and will be ignored.')
+
+    if cfg.zip_archives and cfg.unbox:
+        warning(f'The --zip_archives option was specified, but does nothing when unboxing and will be ignored.')
+
+    if cfg.unbox_multi and not cfg.unbox:
+        warning(f'The --unbox_multi option was specified, but does nothing unless unboxing and will be ignored.')
 
     if cfg.zip_structure and cfg.create_folders:
         warning(f'Keeping sub-folders from root in archives, as well creating the folder structure in the '
@@ -209,12 +246,17 @@ def cli_entry_point():
 
     output_modes = {
         'd': 'd', 'default': 'd',
+        'l': 'l', 'list': 'l',
         'q': 'q', 'quiet': 'q',
         's': 's', 'status': 's',
         'v': 'v', 'verbose': 'v'
     }
+    if cfg.verbose:
+        if 'output' in cfg.arguments:
+            warning(f'Setting --verbose to True overrides any --output options with \'verbose\'.')
+        cfg.output = 'v'
     if cfg.output not in output_modes:
-        error(f'Unknown output mode {cfg.output}, provide default(d), quiet(q), status(s) or verbose(v).')
+        error(f'Unknown output mode {cfg.output}, provide default(d), list(l), quiet(q), status(s) or verbose(v).')
         print_help()
         exit(1)
     cfg.output = output_modes[cfg.output]
